/***************************************************************************
 *   Copyright (C) 2008 by Lisandro Damián Nicanor Pérez Meyer             *
 *   perezmeyer@gmail.com                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef DATAHOLDER_H
#define DATAHOLDER_H

#include <QVector>
#include "point4f.h"

/// Data Holder
/**
  This class holds the data used to plot the surfaces. It also produces the
  vertex arrays to pass them to OpenGL.\n
*/

class DataHolder
{
public:
	DataHolder(int rows, int columns);
	~DataHolder();

	/// Add data
	void addData(int row, int column, const Point4F & data);

	/// Generate a QVector to use as a vertex array
	QVector<double> * getVertexArray(int row) const;

	/// Generate a QVector to use as a normal array
	QVector<double> * getNormalArray(int row) const;

	/// Return the number of rows
	int rows() {return numberOfRows;}
	/// Return the number of columns
	int columns() {return numberOfColumns;}


private:
	/// Calculates the normal of two vectors (cross product)
	Point4F calculateNormal(Point4F first, Point4F second) const;

	/// Array of Point4F used as a matrix
	Point4F * matrix;
	/// Number of rows in the matrix
	int numberOfRows;
	/// Number of columns in the matrix
	int numberOfColumns;
};


#endif // DataHolder


