/***************************************************************************
 *   Original:                                                             *
 *   Copyright (C) 2007 by Pablo Odorico                                   *
 *   pablo.odorico@gmail.com                                               *
 *   Modifications:                                                        *
 *   Copyright (C) 2008 Lisandro Damián Nicanor Pérez Meyer                *
 *   perezmeyer@gmail.com                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "glslutils.h"
#include <QDebug>

bool GLSLUtils::checkSupport(bool verbose)
{
	glewInit();

	bool ret= true;
	if(glewGetExtension("GL_ARB_fragment_shader") != GL_TRUE) {
		if(verbose)
			qDebug() << __func__ << QObject::tr(": GL_ARB_fragment_shader extension is not available");
		ret= false;
	}

	if(glewGetExtension("GL_ARB_vertex_shader") != GL_TRUE) {
		if(verbose)
			qDebug() << __func__ << QObject::tr(": GL_ARB_vertex_shader extension is not available");
		ret= false;
	}

	if(glewGetExtension("GL_ARB_shader_objects") != GL_TRUE) {
		if(verbose)
			qDebug() << __func__ << QObject::tr(": GL_ARB_shader_objects extension is not available");
		ret= false;
	}

	if(verbose)
		qDebug() << __func__ << QObject::tr("Your system supports GLSL");

	return ret;
}

GLenum GLSLUtils::checkOGLError(const QString &file, const int &line)
{
	GLenum glErr = glGetError();
	while(glErr != GL_NO_ERROR) {
		const GLubyte* sError = gluErrorString(glErr);
		qDebug() << "** GL Error number" << glErr << ":" <<	(char *)sError;
	}
	return glErr;
}

bool GLSLUtils::checkCompileError(GLhandleARB object, QString & infoLog,
                                  bool printToConsole)
{
	return checkGLSLError(object, infoLog, true, printToConsole);
}

bool GLSLUtils::checkLinkError(GLhandleARB object, QString & infoLog,
                               bool printToConsole)
{
	return checkGLSLError(object, infoLog, false, printToConsole);
}

bool GLSLUtils::checkGLSLError(GLhandleARB object, QString & infoLog,
                               bool compileError, bool printToConsole)
{
	int status;
	infoLog = QObject::tr("No errors found");

	if(compileError)
		glGetObjectParameterivARB(object, GL_OBJECT_COMPILE_STATUS_ARB, &status);
	else
		glGetObjectParameterivARB(object, GL_OBJECT_LINK_STATUS_ARB, &status);

	// If there is an error
	if(status == 0)
	{
		int logLenght = 0;
		int charsWritten = 0;
		char * infoLogChar = 0;

		// Get the length of the error log
		glGetObjectParameterivARB(object, GL_OBJECT_INFO_LOG_LENGTH_ARB,
		                          &logLenght);

		if(logLenght > 0)
		{
			infoLogChar = (char *)malloc(logLenght);

			// Get the error log
			glGetInfoLogARB(object, logLenght, &charsWritten, infoLogChar);

			// Pass the error to the infoLog QString
			infoLog = QString::fromAscii(infoLogChar);
			free(infoLogChar);
		}

		if(printToConsole)
		{
			if(compileError)
				qDebug() << __func__ << QObject::tr("A GLSL shader compile error has occured");
			else
				qDebug() << __func__ << QObject::tr("A GLSL shader link error has occured");

			qDebug() << infoLog;
		}

		return true;
	}
	return false;
}

