/***************************************************************************
 *   Original:                                                             *
 *   Copyright (C) 2007 by Pablo Odorico                                   *
 *   pablo.odorico@gmail.com                                               *
 *   Modifications:                                                        *
 *   Copyright (C) 2008 Lisandro Damián Nicanor Pérez Meyer                *
 *   perezmeyer@gmail.com                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef GLSLUTILS_H
#define GLSLUTILS_H

#include <GL/glew.h>

#define CHECK_GL_ERROR() GLSLUtils::checkOGLError(__FILE__, __LINE__)

class QString;

class GLSLUtils {
public:
	/// Checks for GLSL support in the system
	static bool checkSupport(bool verbose=false);

	/// Checks for OpenGL errors
	static GLenum checkOGLError(const QString &file, const int &line);

	/// Checks for GLSL shaders compile errors
	/** Returns true if an error occurs */
	static bool checkCompileError(GLhandleARB object, QString & infoLog,
	                              bool printToConsole=false);

	/// Checks for GLSL link errors
	/** Returns true if an error occurs */
	static bool checkLinkError(GLhandleARB object, QString & infoLog,
	                           bool printToConsole=false);

private:
	/// Checks for GLSL compiling and linking errors
	/**
	Returns true if an error occurs. \n
	\param compileError true is we are searching for compiling errors,
	false if we search for linking errors.
	*/
	static bool checkGLSLError(GLhandleARB object, QString & infoLog,
	                    bool compileError, bool printToConsole=false);
};

#endif
